/******************************************************  Flash Header  *******************************************************

	Red Rapids
	797 N Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*******************************************************  Flash Header  ******************************************************/

#ifndef _FLASH_H_
#define _FLASH_H_

#if defined(__cplusplus)
	extern "C" {
#endif

// Number of bytes per register:
#define BYTES 8

// BAR0 register names:
#define SOFTWARE_RESET 0x0000/BYTES
#define ERROR_STATUS 0x0010/BYTES
#define BPI_IF_CTL 0x0040/BYTES
#define BPI_BUS 0x0048/BYTES
#define GLOBAL_INT_MASK 0x0108/BYTES

// BAR2 register names:
#define IDENTIFICATION 0x0000/BYTES

// Flash commands:
#define BLANK_CHECK 0xBC
#define BLOCK_ERASE 0x20
#define BUFFERED_PROGRAM 0xE9
#define CONFIRM_COMMAND 0xD0
#define UNLOCK_BLOCK 0x60
#define READ_ARRAY 0xFF
#define READ_STATUS_REGISTER 0x70
#define CLEAR_STATUS_REGISTER 0x50
#define PROG_READ_CONFIG_REG1 0x60
#define PROG_READ_CONFIG_REG2 0x03
#define READ_ID 0x90

// Programmer commands:
#define ENABLE_COMMAND 0x00000000
#define SEGMENT_COMMAND 0x10000000
#define ADDRESS_COMMAND 0x20000000
#define WRITE_COMMAND 0x30000000
#define READ_COMMAND 0x40000000

// Constants set by the hardware.
#define BLOCK_SIZE 0x20000
#define BLOCK_COUNT 0x100
#define BUFFER_SIZE 0x200
#define PARTITION_SIZE 0x400000
#define PARTITION_COUNT 0x8
#define SEGMENT_COUNT 0x4
#define CMD_FIFO_BURST 0x100
#define TYP_ERASE_TIME 900000 //useconds
#define TYP_BUFPROG_TIME 1000 //useconds
#define TYP_BLNKCHK_TIME 3200 //useconds

#ifdef __cplusplus
}
#endif

#endif
