/*************************************************  Global Functions Header  ***************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*************************************************  Global Functions Header  ***************************************************/

#ifndef _GLOBAL_FUNCTIONS_H_
#define _GLOBAL_FUNCTIONS_H_

#if defined(__cplusplus)
	extern "C" {
#endif


// Structure to store the LED on/off control settings.
typedef struct
{
	unsigned LEDA;
	unsigned LEDB;
} s_LEDControl;

// Structure to store GPIO and USER signal translator directions.
typedef struct
{
	unsigned GPIODirection[6];
	unsigned USERDirection[62];
} s_PinDirection;

// Structure to store the results returned by an error status query to the device.
typedef struct
{
	unsigned Bar[4];
	unsigned Clock[8];
	unsigned Alert;
	unsigned Fan;
} s_ErrorStatus;

// Structure to store the results returned by an interrupt status query to the device.
typedef struct
{
	unsigned Channel[8];
	unsigned Error;
} s_GlobalStatus;

// Structure to store temperature results measured on the device.
typedef struct
{
	double Sensor[5];
	double Vcc;
} s_Temperature;

// Structure to store power results measured on the device.
typedef struct
{
	double Supply1[3];
	double Supply2[3];
	double Supply3[3];
} s_Power;

// Prototypes for each of the global functions in alphabetic order.
unsigned ChannelCount(s_Handle *p_HW, unsigned RxTxSelect);
unsigned ClockFrequency(s_Handle *p_HW);
int ClockInit(s_Handle *p_HW);
int ClockInitLMK01000(s_Handle *p_HW, unsigned CLKout0, unsigned CLKout4, unsigned CLKout5);
int ClockStatus(s_Handle *p_HW);
void ErrorMask(s_Handle *p_HW, s_ErrorStatus ErrorIntMask);
int ErrorStatus(s_Handle *p_HW, s_ErrorStatus *p_Error);
unsigned FirmwareRevision(s_Handle *p_HW);
int GlobalStatus(s_Handle *p_HW, s_GlobalStatus *p_Global);
void GPIOConnect(s_Handle *p_HW);
void GPIODisconnect(s_Handle *p_HW);
void InterruptMask(s_Handle *p_HW, unsigned GlobalIntMask);
int MicrosecondTimer(long uSeconds);
unsigned ModelNumber(s_Handle *p_HW);
void PinDirection(s_Handle *p_HW, s_PinDirection Pin);
double PowerStatus(s_Handle *p_HW, s_Power *p_Power);
int SerDesInit(s_Handle *p_HW);
void Serial(s_Handle *p_HW, unsigned BusSelect, unsigned MultiFunction, unsigned Instruction, unsigned AddressSize, unsigned Data, unsigned DataSize, unsigned *p_ReadValue);
int SigFPGAID(s_UIOScan UIO, s_Handle *p_HW);
void SoftwareReset(s_Handle *p_HW);
void SoftwareTrigger(s_Handle *p_HW);
void TemperatureStatus(s_Handle *p_HW, s_Temperature *p_Temperature);
void USERConnect(s_Handle *p_HW);
void USERDisconnect(s_Handle *p_HW);

#ifdef __cplusplus
}
#endif

#endif
