/*****************************************************  SigFPGA Header  ******************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************  SigFPGA Header  *****************************************************/

#ifndef _SIGFPGA_H_
#define _SIGFPGA_H_

#if defined(__cplusplus)
	extern "C" {
#endif

// The following user defined structure is used to demonstrate the transfer of data between DMA buffers and the application.
// Note: It can be modified to suit a specific application. 
typedef struct
{
	unsigned RXChannelCount;
	unsigned RXDataItemSize;
	unsigned TXChannelCount;
	unsigned TXDataItemSize;
	unsigned **Error;
	void ***p_DataBuffer;
} s_User;

////////////////////////////////////////////////////////  Error Codes  ////////////////////////////////////////////////////////

// Define error codes returned by API functions.
#define OK					0
#define BAD_SETTING			-500
#define MODEL_NUMBER			BAD_SETTING - 1
#define DAC_INTERPOLATE			BAD_SETTING - 2
#define DAC_PLL					BAD_SETTING - 3
#define DMA_BUFFER				BAD_SETTING - 4
#define PHASE_BREAK				BAD_SETTING - 5
#define CONFIG_FAIL			-520
#define CLOCK_NOSOL				CONFIG_FAIL - 1
#define CLOCK_LOCK				CONFIG_FAIL - 2
#define SYNTH_LOCK				CONFIG_FAIL - 3
#define SERDES_NOSOL			CONFIG_FAIL - 4
#define DAC_PLL_LOCK			CONFIG_FAIL - 5
#define DAC_DLL_LOCK			CONFIG_FAIL - 6
#define DAC_FIFO_RST			CONFIG_FAIL - 7
#define RUN_FAIL			-530
#define TIME_OUT				RUN_FAIL - 1
#define FLASH_ERROR				RUN_FAIL - 2

/////////////////////////////////////////////////////  SigFPGA Constants  /////////////////////////////////////////////////////

// DMA transaction limits:
#define MAX_BURSTSIZEB 16383
#define BYTE_ALIGNMENT 16
#define MAX_BURSTCOUNT 65535
#define MAX_PAGECOUNT 1024

// Number of bytes per register:
#define BYTES 8

// BAR0 register names:
#define SOFTWARE_RESET 0x0000/BYTES
#define CLOCK_INITIALIZE 0x0008/BYTES
#define ERROR_STATUS 0x0010/BYTES
#define ERROR_INT_MASK 0x0018/BYTES
#define GLOBAL_STATUS 0x0100/BYTES
#define GLOBAL_INT_MASK 0x0108/BYTES
#define GPIO_CONFIG 0x0050/BYTES
#define INTERRUPT_LATENCY 0x0110/BYTES
#define INTERRUPT_TIMEOUT 0x0118/BYTES
#define SERIAL_PAYLOAD 0x0020/BYTES
#define SERIAL_CONFIG 0x0030/BYTES
#define SERIAL_COMMAND 0x0038/BYTES
#define USER_CONFIG 0x0058/BYTES
#define CLOCK_FREQUENCY 0x0060/BYTES
#define DMA_1TO2_STATUS 0x1F00/BYTES
#define DMA_3TO4_STATUS 0x1F08/BYTES
#define DMA_5TO6_STATUS 0x1F10/BYTES
#define DMA_7TO8_STATUS 0x1F18/BYTES
#define DMA_DESCRIPTORS 0x2000/BYTES

// BAR2 register names for functions that are common to all channels:
#define IDENTIFICATION 0x0000/BYTES
#define SOFTWARE_TRIGGER 0x0010/BYTES
#define LED_ON_OFF_CTL 0x0020/BYTES
#define DEMO_REG_LO 0x0030/BYTES
#define DEMO_REG_HI 0x0038/BYTES
#define SRAM_ERROR 0x0040/BYTES
#define SRAM_CMD_STAT 0x0048/BYTES
#define PCI_THROUGHPUT 0x0050/BYTES
#define PCI_LATENCY 0x0058/BYTES

// BAR2 register names for functions that are unique to each channel:
// Note: This is the register base address from a predefined offset for each channel.
#define SYNCHRONIZER_CTL 0x0000/BYTES
#define CHANNEL_STATUS 0x0010/BYTES
#define CHANNEL_INT_MASK 0x0018/BYTES
#define SYNCHRONIZER_CMD 0x0020/BYTES

// Serial bus transaction types:
#define I2C_READ 1
#define I2C_WRITE 0
#define RISING_EDGE 1
#define FALLING_EDGE 0
// Serial bus type select:
// 0 => I2C
// 1 => SPI unidirectional bus (SDI/SDO)
// 2 => SPI chip select port expander (serial shift register)
// 3 => SPI bidirectional bus (SDIO)
// 4 => GPIO/user translator port expander (serial shift register)
// 3 => Programmable synthesizer
#define I2C 0
#define SPI 1
#define MICROWIRE 1
#define SPI_CSB 2
#define SPI_SDIO 3
#define GPIO_USER_DIRECTION 4
#define SYNTH 5
// Payload format by device (instruction size and data size):
#define AD5628_INST 12
#define AD5628_DATA 20
#define AD9142A_INST 16
#define AD9142A_DATA 8
#define AD9512_INST 16
#define AD9512_DATA 8
#define AD9652_INST 16
#define AD9652_DATA 8
#define AD9653_INST 16
#define AD9653_DATA 8
#define ADC12D1600_INST 8
#define ADC12D1600_DATA 16
#define ADS42LB69_INST 8
#define ADS42LB69_DATA 8
#define FXL6408_DATA 8
#define INA3221_DATA 16
#define LMK01000_ADD 4
#define LMK01000_DATA 28
#define LTC1661_INST 4
#define LTC1661_DATA 12
#define LTC2991_DATA 8
// Serial bus chip selects by product:
#define ALL_DESELECT 0xFFFF
#define CMD_STAT1 0x86
#define DEFAULT_CLK_DIST 0xFFFE
#define M370_RX1_ADC 0xFFFE
#define M370_RX1_TRIM_DAC 0xFFFD
#define M371_CLK_DIST 0xFFFE
#define M371_RX1TO2_ADC 0xFFFD
#define M371_RX1_TRIM_DAC 0xFFF7
#define M371_RX2_TRIM_DAC 0xFFEF
#define M372_CLK_DIST 0xFFFE
#define M372_RX1TO2_ADC 0xFFFD
#define M372_TX3TO4_DAC 0xFFFB
#define M372_RX2_TRIM_DAC 0xFFF7
#define M372_RX1_TRIM_DAC 0xFFEF
#define M372_TX3_TRIM_DAC 0xFFDF
#define M372_TX4_TRIM_DAC 0xFFBF
#define M372_CLK_DIST_SYNC 0xFF7F
#define M373_CLK_DIST 0xFFFE
#define M373_RX1TO2_ADC 0xFFFD
#define M373_RX2_TRIM_DAC 0xFFF7
#define M373_RX1_TRIM_DAC 0xFFEF
#define M373_CLK_DIST_SYNC 0xFF7F
#define M376_RX1TO2_ADC 0xFFFE
#define M376_RX1_TRIM_DAC 0xFFFD
#define M376_RX2_TRIM_DAC 0xFFFB
#define M377_CLK_DIST 0xFFFE
#define M377_RX1TO2_ADC 0xFFFD
#define M377_RX3TO4_ADC 0xFFFB
#define M377_RX1_TRIM_DAC 0xFFF7
#define M377_RX2_TRIM_DAC 0xFFEF
#define M377_RX3_TRIM_DAC 0xFFDF
#define M377_RX4_TRIM_DAC 0xFFBF
#define M378_CLK_DIST 0xFFFE
#define M378_RX1TO4_ADC 0xFFFD
#define M378_RX5TO8_ADC 0xFFFB
#define M378_RX1TO4_TRIM_DAC 0xFFF7
#define M378_RX5TO8_TRIM_DAC 0xFFEF
#define M378_CLK_DIST_SYNC 0xFFDF
#define TEMP_MONITOR 0x92
#define VOLT_MONITOR 0x80

#ifdef __cplusplus
}
#endif

#endif
